package com.mortals.xhx.busiz.web;

import cn.hutool.core.codec.Base64;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.util.HttpUtil;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.model.vo.MatterInfo;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.matters.service.MattersService;
import lombok.extern.apachecommons.CommonsLog;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpResponse;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

import static com.mortals.framework.ap.SysConstains.MESSAGE_INFO;
import static com.mortals.framework.ap.SysConstains.PAGEINFO_KEY;

/**
 * 微官网接口
 *
 * @author:
 * @date: 2022/12/14 9:18
 */
@RestController
@Slf4j
@RequestMapping("micro")
public class MicroWebApiController extends BaseJsonBodyController {

    @Autowired
    private MatterService matterService;

    @PostMapping(value = "matter/list")
    @UnAuth
    public Rest<Object> list(@RequestBody MatterQuery query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "查询微官网事项列表";
        int code = VALUE_RESULT_SUCCESS;
        try {
            PageInfo pageInfo = this.buildPageInfo(query);
            Result<MatterInfo> result = matterService.findMicroList(query, pageInfo, context);
            model.put(KEY_RESULT_DATA, result.getList());
            model.put(PAGEINFO_KEY, result.getPageInfo());
            model.put(MESSAGE_INFO, busiDesc + "成功");
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;
    }

    @PostMapping(value = "getPicAndCookie")
    @UnAuth
    public Rest<Map<String,String>> getPicAndCookie(@RequestBody MatterQuery query) {
        log.info("请求查询图片与cookies:{}",JSON.toJSONString(query));
        HashMap<String, String> resultMap = new HashMap<>();
        CookieStore httpCookieStore = new BasicCookieStore();
        HttpClient http = HttpClientBuilder.create().setDefaultCookieStore(httpCookieStore).build();
        HttpGet httpRequest = new HttpGet(query.getUrl());
        HttpResponse httpResponse = null;
        try {
            httpResponse = http.execute(httpRequest);
            byte[] data = EntityUtils.toByteArray(httpResponse.getEntity());
            String encode = Base64.encode(data);
            resultMap.put("base64img","data:image/jpg;base64,"+encode);
            resultMap.put("cookieStr",JSON.toJSONString(httpCookieStore.getCookies()));

        } catch (Exception e) {
            log.error("异常", e);
        }
        return Rest.ok(resultMap);
    }

    protected PageInfo buildPageInfo(MatterQuery query) {
        PageInfo pageInfo = new PageInfo();
        if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getPage())) {
            pageInfo.setCurrPage(query.getPage());
        }

        if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getSize())) {
            pageInfo.setPrePageResult(query.getSize());
        }

        return pageInfo;
    }

}
