package com.mortals.xhx.daemon.task;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.RandomUtil;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.xhx.common.utils.HttpDownloadUtil;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.common.utils.WordUtil;
import com.mortals.xhx.module.matter.model.MatterDatumEntity;
import com.mortals.xhx.module.matter.model.MatterDatumFileEntity;
import com.mortals.xhx.module.matter.model.MatterDatumFileQuery;
import com.mortals.xhx.module.matter.model.MatterDatumQuery;
import com.mortals.xhx.module.matter.service.MatterDatumFileService;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import com.mortals.xhx.module.site.model.SiteMatterEntity;
import com.mortals.xhx.module.site.model.SiteMatterQuery;
import com.mortals.xhx.module.site.service.SiteMatterService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

/**
 * 事项材料附件本地转换
 */
@Slf4j
@Service("MatterDatumConvertTask")
public class MatterDatumConvertTaskImpl implements ITaskExcuteService {

    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private MatterDatumFileService matterDatumFileService;

    @Value("${upload.path}")
    private String filePath;

    public static String word[] = {"doc", "dot", "wps", "wpt", "docx", "dotx", "docm", "dotm"};
    public static String imgs[] = {"jpg", "png", "jpeg", "gif"};
    public static String pdf[] = {"pdf"};

    @Override
    public void excuteTask(ITask task) throws AppException {
        log.info("事项材料处理...");
        SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
        siteMatterQuery.setIsConvert(0);
        List<SiteMatterEntity> siteMatterEntities = siteMatterService.find(siteMatterQuery);
        if(CollectionUtils.isNotEmpty(siteMatterEntities)){
            List<Long> matterIds = new ArrayList<>();
            for(SiteMatterEntity siteMatterEntity:siteMatterEntities){
                matterIds.add(siteMatterEntity.getMatterId());
            }
            MatterDatumQuery matterDatumQuery = new MatterDatumQuery();
            matterDatumQuery.setMatterIdList(matterIds);
            List<MatterDatumEntity> matterDatumEntityList = matterDatumService.find(matterDatumQuery);
            List<Long> convertIdList = new ArrayList<>();
            if(CollectionUtils.isNotEmpty(matterDatumEntityList)){
                for(MatterDatumEntity datumEntity:matterDatumEntityList){
                    MatterDatumFileQuery fileQuery = new MatterDatumFileQuery();
                    fileQuery.setDatumId(datumEntity.getId());
                    List<MatterDatumFileEntity> datumFileEntityList = matterDatumFileService.find(fileQuery);
                    if(CollectionUtils.isNotEmpty(datumFileEntityList)){
                        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
                        String prePath = "file/preview/";
                        String savePath = "file/uploadfile/";
                        for(MatterDatumFileEntity datumFileEntity:datumFileEntityList){
                            if(StringUtils.isEmpty(datumFileEntity.getFileUrl())||StringUtils.isNotEmpty(datumFileEntity.getLocalFileUrl())){
                                continue;
                            }
                            String fileName = datumFileEntity.getFileName();
                            String fileUrl = datumFileEntity.getFileUrl();
                            try {
                                HttpDownloadUtil.downLoadFromUrl(fileUrl,fileName,rootPath + savePath);
                            }catch (Exception e){
                                log.error("DatumId:"+datumFileEntity.getDatumId()+",文件下载失败...",e);
                                continue;
                            }
                            MatterDatumFileEntity update = new MatterDatumFileEntity();
                            update.setId(datumFileEntity.getId());
                            update.setLocalFileUrl(savePath+fileName);
                            String newName = genPreviewPath(rootPath, prePath,  savePath+fileName);
                            update.setPreviewUrl(prePath+newName);
                            matterDatumFileService.update(update);
                        }
                        convertIdList.add(datumEntity.getMatterId());
                    }
                }
            }
            if(convertIdList.size()>0){
                SiteMatterQuery matterQuery = new SiteMatterQuery();
                matterQuery.setMatterIdList(convertIdList);
                List<SiteMatterEntity> matterEntities = siteMatterService.find(matterQuery);
                for(SiteMatterEntity matterEntity:matterEntities){
                    SiteMatterEntity updateEntity = new SiteMatterEntity();
                    updateEntity.setId(matterEntity.getId());
                    updateEntity.setIsConvert(1);
                    siteMatterService.update(updateEntity);
                }
            }
        }
    }

    @Override
    public void stopTask(ITask task) throws AppException {

    }

    private String genPreviewPath(String rootPath, String prePath, String tempPath) {
        //生成样表预览图片
        String samplePath = rootPath + tempPath;
        String filePath = rootPath + prePath;
        File pathDir = new File(filePath);
        if (!pathDir.exists()) {
            pathDir.mkdirs();
        }
        String newName = RandomUtil.randomNumbers(12) + ".jpg";
        String filePathAll = filePath + newName;
        //判断文件类型是否为doc pdf jpg png 等
        String extName = FileUtil.getSuffix(samplePath);
        if (this.isExsitArry(extName, word)) {
            WordUtil.convertWordToJPEG(samplePath, filePathAll);
        } else if (this.isExsitArry(extName, pdf)) {
            WordUtil.pdfToImages(samplePath, filePathAll);
        } else if (this.isExsitArry(extName, imgs)) {
            try {
                com.mortals.framework.util.FileUtil.copyFile(samplePath, filePathAll);
            } catch (FileNotFoundException e) {
                throw new RuntimeException(e);
            }
        }
        return newName;
    }

    public boolean isExsitArry(String chex, String arry[]) {
        for (String ex : arry) {
            if (chex.equalsIgnoreCase(ex)) {
                return true;
            }
        }
        return false;
    }
}
