package com.mortals.xhx.module.apply.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.base.system.user.model.UserEntity;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.module.apply.dao.MatterApplyDao;
import com.mortals.xhx.module.apply.model.MatterApplyDatumEntity;
import com.mortals.xhx.module.apply.model.MatterApplyDatumQuery;
import com.mortals.xhx.module.apply.model.MatterApplyEntity;
import com.mortals.xhx.module.apply.model.MatterApplyQuery;
import com.mortals.xhx.module.apply.model.vo.DatumFileVO;
import com.mortals.xhx.module.apply.model.vo.MatterApplyDatumGroupVO;
import com.mortals.xhx.module.apply.service.ApproverService;
import com.mortals.xhx.module.apply.service.MatterApplyDatumService;
import com.mortals.xhx.module.apply.service.MatterApplyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* MatterApplyService
* 单事项申报 service实现
*
* @author zxfei
* @date 2023-05-23
*/
@Service("matterApplyService")
@Slf4j
public class MatterApplyServiceImpl extends AbstractCRUDServiceImpl<MatterApplyDao, MatterApplyEntity, Long> implements MatterApplyService {

    @Autowired
    private MatterApplyDatumService matterApplyDatumService;
    @Autowired
    private ApproverService approverService;
    @Autowired
    private UserService userService;

    @Override
    protected void saveBefore(MatterApplyEntity entity, Context context) throws AppException {
        entity.setSiteId(1l);
        entity.setApplyTime(entity.getCreateTime());
        entity.setApplyState(0);
    }

    @Override
    protected void saveAfter(MatterApplyEntity entity, Context context) throws AppException {
        if(CollectionUtils.isNotEmpty(entity.getApplyDatumList())){
            for (MatterApplyDatumEntity item:entity.getApplyDatumList()){
                item.setSiteId(entity.getSiteId());
                item.setSingleMatterId(entity.getSingleMatterId());
                item.setMatterId(entity.getMatterId());
                item.setApplyId(entity.getId());
                item.setCreateTime(entity.getCreateTime());
                item.setCreateUserId(entity.getCreateUserId());
            }
            matterApplyDatumService.save(entity.getApplyDatumList(),context);
        }
    }

    @Override
    protected void updateBefore(MatterApplyEntity entity, Context context) throws AppException {
        entity.setApplyTime(new Date());
        entity.setAdvisement("");
        entity.setApplyState(0);
    }

    @Override
    protected MatterApplyEntity findBefore(MatterApplyEntity params, Context context) throws AppException {
        if(params.getUserId()!=null){
            String deptCode = approverService.getDeptCodeByUser(params.getUserId());
            params.setDeptCode(deptCode);
        }
        MatterApplyQuery query = new MatterApplyQuery();
        BeanUtils.copyProperties(params,query);
        if(StringUtils.isNotEmpty(params.getKeywords())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(params.getKeywords()).append("%");
            MatterApplyQuery condition1 = new MatterApplyQuery();
            condition1.setApplyPerson(condition.toString());
            MatterApplyQuery condition2 = new MatterApplyQuery();
            condition2.setMatterName(condition.toString());
            MatterApplyQuery condition3 = new MatterApplyQuery();
            condition3.setAddress(condition.toString());
            List<MatterApplyQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            orConditionList.add(condition3);
            query.setOrConditionList(orConditionList);
        }
        return query;
    }
    @Override
    protected MatterApplyEntity findBefore(MatterApplyEntity params, PageInfo pageInfo, Context context) throws AppException {
        if(params.getUserId()!=null){
            String deptCode = approverService.getDeptCodeByUser(params.getUserId());
            params.setDeptCode(deptCode);
        }
        MatterApplyQuery query = new MatterApplyQuery();
        BeanUtils.copyProperties(params,query);
        if(StringUtils.isNotEmpty(params.getKeywords())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(params.getKeywords()).append("%");
            MatterApplyQuery condition1 = new MatterApplyQuery();
            condition1.setApplyPerson(condition.toString());
            MatterApplyQuery condition2 = new MatterApplyQuery();
            condition2.setMatterName(condition.toString());
            MatterApplyQuery condition3 = new MatterApplyQuery();
            condition3.setAddress(condition.toString());
            List<MatterApplyQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            orConditionList.add(condition3);
            query.setOrConditionList(orConditionList);
        }
        return query;
    }

    @Override
    protected void updateAfter(MatterApplyEntity entity, Context context) throws AppException {
        if(CollectionUtils.isNotEmpty(entity.getApplyDatumList())){
            matterApplyDatumService.update(entity.getApplyDatumList());
        }
    }

    @Override
    protected void findAfter(MatterApplyEntity params, Context context, List<MatterApplyEntity> list) throws AppException {
        if(CollectionUtils.isNotEmpty(list)){
            for (MatterApplyEntity entity:list){
                List<MatterApplyDatumEntity> datumEntities = matterApplyDatumService.find(new MatterApplyDatumQuery().applyId(entity.getId()));
                entity.setApplyDatumList(datumEntities);
                entity.setDatumGroupList(convertDatum(datumEntities));
                if(entity.getApplyType()==1){
                    entity.setApplyTypeName("移动端申请");
                }else {
                    if(entity.getCreateUserId()!=null) {
                        UserEntity userEntity = userService.get(entity.getCreateUserId());
                        if(userEntity!=null) {
                            entity.setApplyTypeName("后台申请(" + userEntity.getRealName() + ")");
                        }else {
                            entity.setApplyTypeName("后台申请");
                        }
                    }else {
                        entity.setApplyTypeName("后台申请");
                    }
                }
            }
        }
    }

    @Override
    protected void findAfter(MatterApplyEntity params, PageInfo pageInfo, Context context, List<MatterApplyEntity> list) throws AppException {
        if(CollectionUtils.isNotEmpty(list)){
            for (MatterApplyEntity entity:list){
                List<MatterApplyDatumEntity> datumEntities = matterApplyDatumService.find(new MatterApplyDatumQuery().applyId(entity.getId()));
                entity.setApplyDatumList(datumEntities);
                entity.setDatumGroupList(convertDatum(datumEntities));
                if(entity.getApplyType()==1){
                    entity.setApplyTypeName("移动端申请");
                }else {
                    if(entity.getCreateUserId()!=null) {
                        UserEntity userEntity = userService.get(entity.getCreateUserId());
                        if(userEntity!=null) {
                            entity.setApplyTypeName("后台申请(" + userEntity.getRealName() + ")");
                        }else {
                            entity.setApplyTypeName("后台申请");
                        }
                    }else {
                        entity.setApplyTypeName("后台申请");
                    }
                }
            }
        }
    }

    private List<MatterApplyDatumGroupVO> convertDatum(List<MatterApplyDatumEntity> datumEntities){
        List<MatterApplyDatumGroupVO> groupVOList = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(datumEntities)){
            Map<Long, List<MatterApplyDatumEntity>> map = datumEntities.stream().collect(Collectors.groupingBy(t -> t.getDatumId()));
            for (Long key:map.keySet()){
                MatterApplyDatumGroupVO groupVO = new MatterApplyDatumGroupVO();
                BeanUtils.copyProperties(map.get(key).get(0),groupVO);
                List<DatumFileVO> datumFileList = new ArrayList<>();
                for(MatterApplyDatumEntity item:map.get(key)){
                    DatumFileVO fileVO = new DatumFileVO();
                    fileVO.setApplyDatumId(item.getId());
                    fileVO.setFileName(item.getFileName());
                    fileVO.setFileUrl(item.getFileUrl());
                    datumFileList.add(fileVO);
                }
                groupVO.setDatumFileList(datumFileList);
                groupVOList.add(groupVO);
            }
        }
        return groupVOList;
    }

    @Override
    public void applyExamine(MatterApplyEntity entity, Context context) throws AppException {
        if(entity.getId()==null){
            throw new AppException("申报id不能为空");
        }
        if(entity.getApplyState()==null){
            throw new AppException("审批状态不能为空");
        }
        MatterApplyEntity update = new MatterApplyEntity();
        update.setId(entity.getId());
        update.setApplyState(entity.getApplyState());
        update.setAdvisement(entity.getAdvisement());
        if(context!=null && context.getUser()!=null){
            update.setUpdateUserId(context.getUser().getId());
        }
        update.setUpdateTime(new Date());
        dao.update(update);
    }

    @Override
    public void reApply(MatterApplyEntity entity, Context context) throws AppException {
        if(entity.getId()==null){
            throw new AppException("申报id不能为空");
        }
        entity.setApplyState(0);
        entity.setAdvisement("");
        this.update(entity,context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        matterApplyDatumService.remobeByApplyIds(ids,context);
    }
}